%Matlab function to classify the cells in a DEM (Classification.m)
%input is elevation matrix and number of rows and columns.
%output is a classification matrix with 5 different classes
% (1)Complicated  if more than one valley.
% (2)Sink if all cells are higher than the centre cell.
% (3)Flat cells are higher than the centre cell except at least one is equal elevation.
% (4)Peak all cells are lower than the centre cell
% (5)Undisturbed if there is one valley no matter how many cells, max 7 
%Abdulghani 2009-11-4

function [Classification_matrix]=Classification(ZT,nrows,ncols)
%ZT(ZT<0)= NaN; %All - sign are no data NaN
%ZT(ZT==0)= 0.0001; %All 0 elevation are positive data to avoid NaN
Classification_matrix=zeros(nrows,ncols);%creating new matrix filled with zeros 
for r=2:nrows-1 %Go through all cells except the borders
for c=2:ncols-1
    %coordinates and numbering a 3*3 window cells
    %-------------------------
    %| 1 , 9 |   2   |   3   |
    %| 0 , 8 |   10  |   4   |
    %|   7   |   6   |   5   |
    %-------------------------
    %define a 3*3 window cells around the centre cell. duplicate cell8 and
    %it name cell0 also duplicate cell1 and give it number 9
    %the centre cell is number 10.
    Cell0=ZT(r,c-1);  Cell1=ZT(r-1,c-1); Cell2=ZT(r-1,c); Cell3=ZT(r-1,c+1);Cell4=ZT(r,c+1);Cell5=ZT(r+1,c+1);
    Cell6=ZT(r+1,c); Cell7=ZT(r+1,c-1); Cell8=ZT(r,c-1);Cell9=ZT(r-1,c-1); Cell10=ZT(r,c);
    %Putting all 11 cells in a vector matrix
    Cellsmat=[Cell0; Cell1; Cell2; Cell3; Cell4; Cell5; Cell6; Cell7; Cell8; Cell9; Cell10];
    %Cellsmat(isnan(Cellsmat)) = 0; %making all NaNs = 0
 
    ISNAN=isnan(Cellsmat);
if sum(ISNAN) == 0 % Check if there is zero in the vector matrix.
    %since we will classify all cells with positive elevation but not the NaNs
      Sink=1;
      Flat=0;
      Valley=0;
      Peak=0;
for n=2:9 %this is the indexes for the cells 1 to 8
    % Cellsmat(11) is the centre cell value
  if Cellsmat(n) < Cellsmat(11)
    Peak=Peak+1;
    Sink=0;
    Flat=0;
    
     if Cellsmat(n-1) >= Cellsmat(11)%Cellsmat(n-1)is one cell before.
         %if the cell before current cell is greater than OR EQUAL to the
         %center cell then this is a valley(this = sign has been added
         %later to correct classification problem
        Valley=Valley+1;
     end
        
  end
    
    if Cellsmat(n) == Cellsmat(11)%if the current cell equal the centre cell
        Sink=0;
        if Valley == 0
            Flat=Flat+1;
        end
    end
  
end
 
       if Valley > 1 % if more than one valley
   Classification_matrix(r,c)=1;  %Complicated
      
       elseif Sink ==1
               Classification_matrix(r,c)=2;  % sink  
         
       elseif Flat > 0 && Valley ==0
               Classification_matrix(r,c)=3; %Flat
              
       elseif Peak == 8
               Classification_matrix(r,c)=4;  %Peak 
  
       else Classification_matrix(r,c)=5; %Undisturbed 

        end
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end
end
end
Classification_matrix(Classification_matrix == 0)=NaN;
% end of function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%