%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Triangular form basd flow distribution algorithm  
% Matlab function to distribute the flow to the neighboring cells
%The result is 8 matrices showing drainage path from each cell to 8 possible 
%directions.
%Classes (1.Complicated, 2.Sink, 3.Flat, 4.Peak, 5.Undisturbed,)
%
%Abdulghani 2011-09-15
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,Check_Matrix,INan]=FlowDistribution(Classification_matrix,ZT,nrows,ncols)
%% Creating empty matrices
%creating 8 matrices filled with zeros to be used for flow distribution to
%8 different directions
FP1=zeros(nrows,ncols);
FP2=zeros(nrows,ncols);
FP3=zeros(nrows,ncols);
FP4=zeros(nrows,ncols);
FP5=zeros(nrows,ncols);
FP6=zeros(nrows,ncols);
FP7=zeros(nrows,ncols);
FP8=zeros(nrows,ncols);
%create new matrix to mark done cells (initialy zero will be 1 when done) 
Check_Matrix=zeros(nrows,ncols);
%
% INan = isnan(ZT); %identify NaNs
INan=isnan(Classification_matrix);
%give NaN value exist in the elevation matrix to same cells in the flow
%path matrices
FP1(INan)= NaN;
FP2(INan)= NaN;
FP3(INan)= NaN;
FP4(INan)= NaN;
FP5(INan)= NaN;
FP6(INan)= NaN;
FP7(INan)= NaN;
FP8(INan)= NaN;
Check_Matrix(INan)= NaN;
%
% To calculate the number of cells that we need to estimate their flow distribution 
tempmat=imcomplement(Check_Matrix); %NaN will be zero and all zeros will be NAN
Remain_cells=sum(nansum(tempmat)); % calculate number of NaNs
clear tempmat
Check_Matrix(INan)= 1; % Mark NAN cells as done
%%
for r=2:nrows-1
for c=2:ncols-1
    %coordinates and numbering a 3*3 window cells
    %-------------------------
    %| 8  |   1   |   2   |
    %| 7  |   10  |   3   |
    %| 6  |   5   |   4   |
    %-------------------------
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %start with two classes if the cell is complicated or Peak since the
    %same calculations can be used for both
    %...the clssification =5 is now done like 4 and 1. note that 5 is skipt in
    %the original plase where I write 99 instead....
    if Classification_matrix(r,c)== 1 || Classification_matrix(r,c)== 4 || Classification_matrix(r,c)== 5 
     %window of cells around current cell   
    Cell1=ZT(r-1,c); Cell2=ZT(r-1,c+1);Cell3=ZT(r,c+1);Cell4=ZT(r+1,c+1);
    Cell5=ZT(r+1,c); Cell6=ZT(r+1,c-1); Cell7=ZT(r,c-1); Cell8=ZT(r-1,c-1); Cell10=ZT(r,c);
    %Putting all 8 cells in a vector matrix
    cellsmat=[Cell1; Cell2; Cell3; Cell4; Cell5; Cell6; Cell7; Cell8; Cell1; Cell10 ];
    %
%% If there is only one lower cell, skip all calculation and give 100 to
% that cell
cm=0;

for i=1:8
    if cellsmat(i) < Cell10
      cm=cm+1;
      OneCellIndex=i;
        
    end
    
end


%% Calculate the slope and Aspect for each triangle
% Ctreare marix P for the triangle heads coordinates of each cell
P=[0 1 cellsmat(1);
   1 1 cellsmat(2);
   1 0 cellsmat(3);
   1 -1 cellsmat(4);
   0 -1 cellsmat(5);
  -1 -1 cellsmat(6);
  -1 0 cellsmat(7);
  -1 1 cellsmat(8);
   0 1 cellsmat(1);
   0 0 cellsmat(10)];
u=zeros(8,3);

for i=1:8
    ab=P(i,:) - P(10,:);
    ac=P(i+1,:) - P(10,:);
    u(i,:)=cross(ab,ac);
end

%
    %% calculate the aspect % aspect 1 is for the triangel between cells 1 and 2
  asp =zeros(1,8);
  for i=1:8  
 
   if u(i,1)==0 && u(i,2)>=0
    asp(i)=180;
   end
   if u(i,1)==0 && u(i,2)<0
    asp(i)=0;
   end
          
  if u(i,1)>0
      
 asp(i)=rad2deg((3*pi/2) - atan(u(i,2)/u(i,1)));%if u2 or a1 is negative
  end
 if u(i,1)<0
  asp(i)=rad2deg((pi/2) - atan(u(i,2)/u(i,1))); %if u2 or a1 is positive
 end 
%
  end        
%
% if r==66 && c==293
%      asp
%      pause
%  end
 %% Area of small triangles
 smallTriangle=zeros(1,8);
 smallTriangle=(1/8)+smallTriangle;
 %% copy the smallTri
% then we will have original copy and an updated one with the changes from each cell 
%smallTriangle=smallTri;
%smallTriangle=zeros(1,8);
%% Split to two triangles
%check each cell and mark them with SPLIT2 
 SPLIT2=zeros(8,1);
%#1
 if asp(1)>180 && asp(1) <225 
SPLIT2(1)=1;
 end
 
 %#2
 if asp(2)>225 && asp(2) <270
SPLIT2(2)=1;
 end
 
 %#3
 if asp(3)>270 && asp(3) <315 
SPLIT2(3)=1;
 end
 
 if asp(4)>315 && asp(4) <360 
SPLIT2(4)=1;
 end
 
 if asp(5)>0 && asp(5) <45 
SPLIT2(5)=1;
 end
 
 if asp(6)>45 && asp(6) <90 
SPLIT2(6)=1;
 end
 
 if asp(7)>90 && asp(7) <135 
SPLIT2(7)=1;
 end
 
 if asp(8)>135 && asp(8) <180
SPLIT2(8)=1;
 end
 %%%%%%%%%%%%%%%%%%%
%  if r==66 && c==293
%     SPLIT2 
%      pause
%  end
 %%%%%%%%%%%%%%%%%%%%%
 
 if sum(SPLIT2)>0
  %spliting2
  if SPLIT2(1)==1
      if (SPLIT2(2)==1 && SPLIT2(8)==1) || (SPLIT2(2)==0 && SPLIT2(8)==0)
 FI=asp(1)-180;
          [Wa,Wc]=VectorSplit45(FI); 
                
        smallTriangle(8)= smallTriangle(8) + (smallTriangle(1)*Wa)/100;
        smallTriangle(2)= smallTriangle(2) + (smallTriangle(1)*Wc)/100;
        smallTriangle(1)=0;
      end
  end
 
  if SPLIT2(2)==1
      if (SPLIT2(1)==1 && SPLIT2(3)==1) || (SPLIT2(1)==0 && SPLIT2(3)==0)
 FI=asp(2)-225;
          [Wa,Wc]=VectorSplit45(FI); 
                
        smallTriangle(1)= smallTriangle(1) + (smallTriangle(2)*Wa)/100;
        smallTriangle(3)= smallTriangle(3) + (smallTriangle(2)*Wc)/100;
        smallTriangle(2)=0;
      end
  end
  
 if SPLIT2(3)==1
     if (SPLIT2(2)==1 && SPLIT2(4)==1) || (SPLIT2(2)==0 && SPLIT2(4)==0)
 FI=asp(3)-270;
          [Wa,Wc]=VectorSplit45(FI); 
                
        smallTriangle(2)= smallTriangle(2) + (smallTriangle(3)*Wa)/100;
        smallTriangle(4)= smallTriangle(4) + (smallTriangle(3)*Wc)/100;
        smallTriangle(3)=0;
     end
  end 
  
  if SPLIT2(4)==1
      if (SPLIT2(3)==1 && SPLIT2(5)==1) || (SPLIT2(3)==0 && SPLIT2(5)==0)
 FI=asp(4)-315;
          [Wa,Wc]=VectorSplit45(FI); 
                
        smallTriangle(3)= smallTriangle(3) + (smallTriangle(4)*Wa)/100;
        smallTriangle(5)= smallTriangle(5) + (smallTriangle(4)*Wc)/100;
        smallTriangle(4)=0;
      end
  end 
  
  if SPLIT2(5)==1
      if (SPLIT2(4)==1 && SPLIT2(6)==1)|| (SPLIT2(4)==0 && SPLIT2(6)==0)
 FI=asp(5);
          [Wa,Wc]=VectorSplit45(FI); 
                
        smallTriangle(4)= smallTriangle(4) + (smallTriangle(5)*Wa)/100;
        smallTriangle(6)= smallTriangle(6) + (smallTriangle(5)*Wc)/100;
        smallTriangle(5)=0;
      end
  end 
  
 if SPLIT2(6)==1
     if (SPLIT2(5)==1 && SPLIT2(7)==1) || (SPLIT2(5)==0 && SPLIT2(7)==0)
 FI=asp(6)-45;
          [Wa,Wc]=VectorSplit45(FI); 
                
        smallTriangle(5)= smallTriangle(5) + (smallTriangle(6)*Wa)/100;
        smallTriangle(7)= smallTriangle(7) + (smallTriangle(6)*Wc)/100;
        smallTriangle(6)=0;
     end
  end 
  
 if SPLIT2(7)==1
     if (SPLIT2(6)==1 && SPLIT2(8)==1) || (SPLIT2(6)==0 && SPLIT2(8)==0)
 FI=asp(7)-90;
          [Wa,Wc]=VectorSplit45(FI); 
              
        smallTriangle(6)= smallTriangle(6) + (smallTriangle(7)*Wa)/100;
        smallTriangle(8)= smallTriangle(8) + (smallTriangle(7)*Wc)/100;
        smallTriangle(7)=0;
        
     end
  end  
  
 if SPLIT2(8)==1
     if (SPLIT2(7)==1 && SPLIT2(1)==1) || (SPLIT2(7)==0 && SPLIT2(1)==0)
 FI=asp(8)-135;
          [Wa,Wc]=VectorSplit45(FI); 
           
        smallTriangle(7)= smallTriangle(7) + (smallTriangle(8)*Wa)/100;
        smallTriangle(1)= smallTriangle(1) + (smallTriangle(8)*Wc)/100;
        smallTriangle(8)=0;
        
     end
       %there is bug here 
 end
 end %end if sum(SPLIT2)>0
 %%%%%%%%%%%%%%%%%%%
% if r==66 && c==293
%     p1=1
%     smallTriangle
%      pause
%  end
 %%
 if sum(SPLIT2)>0
  %spliting2 case 2
 if SPLIT2(1)==1 && smallTriangle(1) > 0
  if SPLIT2(2)==1 && SPLIT2(8)==0 
    smallTriangle(8)= smallTriangle(8) + smallTriangle(1);
       smallTriangle(1)=0;   
  end
  if SPLIT2(2)==0 && SPLIT2(8)==1 
    smallTriangle(2)= smallTriangle(2) + smallTriangle(1);
       smallTriangle(1)=0;   
  end
end

if SPLIT2(2)==1 && smallTriangle(2) > 0
  if SPLIT2(1)==1 && SPLIT2(3)==0 
    smallTriangle(3)= smallTriangle(3) + smallTriangle(2);
       smallTriangle(2)=0;   
  end
  if SPLIT2(1)==0 && SPLIT2(3)==1 
    smallTriangle(1)= smallTriangle(1) + smallTriangle(2);
       smallTriangle(2)=0;   
  end
end

if SPLIT2(3)==1 && smallTriangle(3) > 0
  if SPLIT2(2)==1 && SPLIT2(4)==0 
    smallTriangle(4)= smallTriangle(4) + smallTriangle(3);
       smallTriangle(3)=0;   
  end
  if SPLIT2(2)==0 && SPLIT2(4)==1 
    smallTriangle(2)= smallTriangle(2) + smallTriangle(3);
       smallTriangle(3)=0;   
  end
end

if SPLIT2(4)==1 && smallTriangle(4) > 0
  if SPLIT2(3)==1 && SPLIT2(5)==0 
    smallTriangle(5)= smallTriangle(5) + smallTriangle(4);
       smallTriangle(4)=0;   
  end
  if SPLIT2(3)==0 && SPLIT2(5)==1 
    smallTriangle(3)= smallTriangle(3) + smallTriangle(4);
       smallTriangle(4)=0;   
  end
end

if SPLIT2(5)==1 && smallTriangle(5) > 0
  if SPLIT2(4)==1 && SPLIT2(6)==0 
    smallTriangle(6)= smallTriangle(6) + smallTriangle(5);
       smallTriangle(5)=0;   
  end
  if SPLIT2(4)==0 && SPLIT2(6)==1 
    smallTriangle(4)= smallTriangle(4) + smallTriangle(5);
       smallTriangle(5)=0;   
  end
end

if SPLIT2(6)==1 && smallTriangle(6) > 0
  if SPLIT2(5)==1 && SPLIT2(7)==0 
    smallTriangle(7)= smallTriangle(7) + smallTriangle(6);
       smallTriangle(6)=0;   
  end
  if SPLIT2(5)==0 && SPLIT2(7)==1 
    smallTriangle(5)= smallTriangle(5) + smallTriangle(6);
       smallTriangle(6)=0;   
  end
end

if SPLIT2(7)==1 && smallTriangle(7) > 0
  if SPLIT2(6)==1 && SPLIT2(8)==0 
    smallTriangle(8)= smallTriangle(8) + smallTriangle(7);
       smallTriangle(7)=0;   
  end
  if SPLIT2(6)==0 && SPLIT2(8)==1 
    smallTriangle(6)= smallTriangle(6) + smallTriangle(7);
       smallTriangle(7)=0;   
  end
end

if SPLIT2(8)==1 && smallTriangle(8) > 0
  if SPLIT2(7)==1 && SPLIT2(1)==0 
    smallTriangle(1)= smallTriangle(1) + smallTriangle(8);
       smallTriangle(8)=0;   
  end
  if SPLIT2(7)==0 && SPLIT2(1)==1 
    smallTriangle(7)= smallTriangle(7) + smallTriangle(8);
       smallTriangle(8)=0;   
  end
end
 end %end if sum(SPLIT2)>0
 %%%%%%%%%%%%%%%%%%%
% if r==66 && c==293
%     p1=2
%     smallTriangle
%      pause
%  end
 
 %% Move water from the higher elevation smalltringel
 % this will move the area and added to adjecent tringle and to check for
 % contradictive
 Continue=1;
 move12=0; move23=0; move34=0; move45=0; move56=0; move67=0; move78=0; move81=0;
 move18=0; move21=0; move32=0; move43=0; move54=0; move65=0; move76=0; move87=0;

%Check the MOVE direction for each cell
 MOVE=zeros(8,1);
%#1
 if asp(1)>=90 && asp(1) <=180 % case 3 odd
MOVE(1)=2;

 end

 if asp(1)>=225 && asp(1) <=270 % case 5 odd
     MOVE(1)=8;
 end
 
  %pause
 %#2
 if asp(2)>=180 && asp(2) <=225 % case 5 even
     MOVE(2)=3;
 end

 %pause
 if (asp(2)>=270 && asp(2) <=360) || asp(2) ==0 % case 3 even
     MOVE(2)=1;
 end

  %pause
 %#3
 if asp(3)>=180 && asp(3) <=270 %case 3 odd
     MOVE(3)=4;

 end

 %pause
 if (asp(3)>=315 && asp(3) <=360) || asp(3) ==0 % case 5 odd
     MOVE(3)=2;
 end

  %pause
  %#4
 if asp(4)>=270 && asp(4) <=315 % case 5 even
     MOVE(4)=5;
 end
 
 %pause
 if asp(4)<=90 % case 3 even
     MOVE(4)=3;
 end
 
  %pause
 %#5
 if asp(5)>=270 || asp(5) ==0
     MOVE(5)=6;
 end
 
 %pause
 if asp(5) >=45 && asp(5) <= 90
     MOVE(5)=4;
 end
 
  %pause
 %#6
 if asp(6) <=45
     MOVE(6)=7;
 end
 
 %pause
 if asp(6)>=90 && asp(6) <=180
     MOVE(6)=5;
 end
  
  %pause
  %#7
 if asp(7) <=90
     MOVE(7)=8;
 end

 %pause
 if asp(7)>=135 && asp(7) <=180
     MOVE(7)=6;
 end
 
  %pause
 %#8
 if asp(8) >=90 && asp(8) <=135 
     MOVE(8)=1;
 end
 
 %pause
 if asp(8)>=180 && asp(8) <=270
     MOVE(8)=7;
 end
 
  %pause
%%%%%%%%%%%%%%%%%%%
% if r==66 && c==293
%     p1=3
%     MOVE
%     smallTriangle
%      pause
%  end

%%
if sum(MOVE)>0
 while Continue==1
    Continue=0; 

 %#1
 if MOVE(1)==2 && smallTriangle(1)>0 && move21==0
    smallTriangle(2)= smallTriangle(2)+smallTriangle(1);
    smallTriangle(1)=0;
    move12=1;
    Continue=1;
 
 end
 
 
 
 if MOVE(1)==8 && smallTriangle(1)>0 && move81==0
    smallTriangle(8)= smallTriangle(8)+smallTriangle(1);
    smallTriangle(1)=0;
    move18=1;
    Continue=1;
 
 end
 
 
  %pause
 %#2
 
 if MOVE(2)==3 && smallTriangle(2)>0 && move32==0
    smallTriangle(3)= smallTriangle(3)+smallTriangle(2);
    smallTriangle(2)=0;
    move23=1;
    Continue=1;
 
 end
  
 %pause
 if MOVE(2)==1 && smallTriangle(2)>0 && move12==0
    smallTriangle(1)= smallTriangle(1)+smallTriangle(2);
    smallTriangle(2)=0;
    move21=1;
    Continue=1;
 
 end
 
 
  %pause
 %#3
 
 if MOVE(3)==4 && smallTriangle(3)>0 && move43==0
    smallTriangle(4)= smallTriangle(4)+smallTriangle(3);
    smallTriangle(3)=0;
    move34=1;
    Continue=1;
 
 end
 

 %pause
 if MOVE(3)==2 && smallTriangle(3)>0 && move23==0
    smallTriangle(2)= smallTriangle(2)+smallTriangle(3);
    smallTriangle(3)=0;
    move32=1;
    Continue=1;
 
 end
 
 
  %pause
  %#4
  
  if MOVE(4)==5 && smallTriangle(4)>0 && move54==0
    smallTriangle(5)= smallTriangle(5)+smallTriangle(4);
    smallTriangle(4)=0;
    move45=1;
    Continue=1;
 
  end
  
 
 %pause
 
 if MOVE(4)==3 && smallTriangle(4)>0 && move34==0
    smallTriangle(3)= smallTriangle(3)+smallTriangle(4);
    smallTriangle(4)=0;
    move43=1;
    Continue=1;
 
  end
 
 
  %pause
 %#5
 
 if MOVE(5)==6 && smallTriangle(5)>0 && move65==0
    smallTriangle(6)= smallTriangle(6)+smallTriangle(5);
    smallTriangle(5)=0;
    move56=1;
    Continue=1;
 
 end
  
 %pause
 
 if MOVE(5)==4 && smallTriangle(5)>0 && move45==0 
    smallTriangle(4)= smallTriangle(4)+smallTriangle(5);
    smallTriangle(5)=0;
    move54=1;
    Continue=1;
 
 end
 
  %pause
 %#6
 
 if MOVE(6)==7 && smallTriangle(6)>0 && move76==0
    smallTriangle(7)= smallTriangle(7)+smallTriangle(6);
    smallTriangle(6)=0;
    move67=1;
    Continue=1;
 
 end
 
 
 %pause
 
 if MOVE(6)==5 && smallTriangle(6)>0 && move56==0
    smallTriangle(5)= smallTriangle(5)+smallTriangle(6);
    smallTriangle(6)=0;
    move65=1;
    Continue=1;
 end
  
  
  %pause
  %#7
  
  if MOVE(7)==8 && smallTriangle(7)>0 && move87==0
    smallTriangle(8)= smallTriangle(8)+smallTriangle(7);
    smallTriangle(7)=0;
    move78=1;
    Continue=1;
 end
  
 
 
 %pause
 
 if MOVE(7)==6 && smallTriangle(7)>0 && move67==0
    smallTriangle(6)= smallTriangle(6)+smallTriangle(7);
    smallTriangle(7)=0;
    move76=1;
    Continue=1;
 
 end
 
 
  %pause
 %#8
 
 if MOVE(8)==1 && smallTriangle(8)>0 && move18==0
    smallTriangle(1)= smallTriangle(1)+smallTriangle(8);
    smallTriangle(8)=0;
    move81=1;
    Continue=1;
 end
 
 
 %pause
 
 if MOVE(8)==7 && smallTriangle(8)>0 && move78==0
    smallTriangle(7)= smallTriangle(7)+smallTriangle(8);
    smallTriangle(8)=0;
    move87=1;
    Continue=1;
 
 end

 
  %pause
 end %while
 end % end if sum(MOVE)>0
% if r==66 && c==293
%     p1=4
%     MOVE
%     smallTriangle
%      pause
%  end
%% Split triangles
%check the SPLIT direction for each cell
 SPLIT=zeros(8,1);
%#1
 if asp(1)>45 && asp(1) <90 % case 2 odd
SPLIT(1)=2;

 end
 
 if asp(1)>270 && asp(1) <360 % case 6 odd
     SPLIT(1)=8;
 end
 
  %pause
 %#2
 if asp(2)>90 && asp(2) <180 % case 6 even
     SPLIT(2)=3;
 end
 
 %pause
 if asp(2)>0 && asp(2) <45 % case 2 even
     SPLIT(2)=1;
 end
 
  %pause
 %#3
 if asp(3)>135 && asp(3) <180 %case 2 odd
     SPLIT(3)=4;

 end
 
 %pause
 if asp(3)>0 && asp(3) <90 % case 6 odd
     SPLIT(3)=2;
 end
 
  %pause
  %#4
 if asp(4)>180 && asp(4) <270 % case 5 even
     SPLIT(4)=5;
 end
 
 %pause
 if asp(4)>90 && asp(4) <135 % case 3 even
     SPLIT(4)=3;
 end
 
  %pause
 %#5
 if asp(5)>225 && asp(5) <270
     SPLIT(5)=6;
 end
 
 %pause
 if asp(5) >90 && asp(5) <180
     SPLIT(5)=4;
 end
 
  %pause
 %#6
 if asp(6) >270 && asp(6) <360
     SPLIT(6)=7;
 end
 
 %pause
 if asp(6)>180 && asp(6) <225
     SPLIT(6)=5;
 end
 
  %pause
  %#7
 if asp(7) >315 && asp(7) <360
     SPLIT(7)=8;
 end
 
 %pause
 if asp(7)>180 && asp(7) <270
     SPLIT(7)=6;
 end

  %pause
 %#8
 if asp(8) >0 && asp(8) <90
     SPLIT(8)=1;
 end
 
 %pause
 if asp(8)>270 && asp(8) <315
     SPLIT(8)=7;
 end
 
  %pause
  %%    
      if SPLIT(8)==1 && SPLIT(1)==8
          SPLIT(8)=0;  SPLIT(1)=0;
      end
      for i=1:7
          
          if SPLIT(i)==i+1 && SPLIT(i+1)==i
          SPLIT(i)=0;  SPLIT(i+1)=0;
          end
          
      end
  %%
%smallTriangle=smallTriangle;

 %% Split the triangle area between two triangles
 % for odd cells
 SumSPLIT=sum(SPLIT);
 while SumSPLIT > 0
 
 
 if SPLIT(1)==2 && SPLIT(8)~=1
   FI=asp(1)-45;
          [Wa,Wc]=VectorSplit45(FI); 
%            
                
        smallTriangle(2)= smallTriangle(2)+ (smallTriangle(1)*Wa)/100;
        smallTriangle(1)= (smallTriangle(1)*Wc)/100;
        SPLIT(1)=0;
        %smallTriangle(i)=0;
 end   
 
 %%%%%
 if SPLIT(3)==4 && SPLIT(2)~=3
   FI=asp(3)-135;
          [Wa,Wc]=VectorSplit45(FI); 
%            
                
        smallTriangle(4)= smallTriangle(4)+ (smallTriangle(3)*Wa)/100;
        smallTriangle(3)= (smallTriangle(3)*Wc)/100;
        SPLIT(3)=0;
        %smallTriangle(i)=0;
 end   
 
 
 %%%%
 if SPLIT(5)==6 && SPLIT(4)~=5
   FI=asp(5)-225;
          [Wa,Wc]=VectorSplit45(FI); 
%            
                
        smallTriangle(6)= smallTriangle(6)+ (smallTriangle(5)*Wa)/100;
        smallTriangle(5)= (smallTriangle(5)*Wc)/100;
        SPLIT(5)=0;
        %smallTriangle(i)=0;
 end   
 %%%%
 
 if SPLIT(7)==8 && SPLIT(6)~=7
   FI=asp(7)-315;
          [Wa,Wc]=VectorSplit45(FI); 
%            
                
        smallTriangle(8)= smallTriangle(8)+ (smallTriangle(7)*Wa)/100;
        smallTriangle(7)= (smallTriangle(7)*Wc)/100;
        SPLIT(7)=0;
        %smallTriangle(i)=0;
 end   
 
 
%  smallTriangle
%  ss=1
%  sum(smallTriangle)
 %
 % for odd cells other direction
 
 if SPLIT(1)==8 && SPLIT(2)~=1
   
     FI=asp(1)-270;
      [Wa,Wc]=VectorSplit90(FI);  
%             A=FI*pi/180;
%         AA=(90-FI)*pi/180;
%         vector_a = (sin(A))/(sin(AA));
%         Wa=vector_a/(vector_a +1);
%         Wa=(fix(Wa*10000))/100;
%         Wc=100-Wa;
        
        smallTriangle(8)= smallTriangle(8)+ (smallTriangle(1)*Wc)/100;
        smallTriangle(1)= (smallTriangle(1)*Wa)/100;
        SPLIT(1)=0;
        %smallTriangle(1)=0;
 end
% smallTriangle
% ss=2
% sum(smallTriangle)
 if SPLIT(3)==2 && SPLIT(4)~=3
      FI=asp(3);
      [Wa,Wc]=VectorSplit90(FI);  
                
        smallTriangle(2)= smallTriangle(2)+(smallTriangle(3) *Wc)/100;
        smallTriangle(3)= (smallTriangle(3)*Wa)/100;
        SPLIT(3)=0;
        %smallTriangle(3)=0;
 end
%  smallTriangle
%  ss=3
%  sum(smallTriangle)
 if SPLIT(5)==4 && SPLIT(6)~=5
      FI=asp(5)-90;
      [Wa,Wc]=VectorSplit90(FI);  
        
        smallTriangle(4)= smallTriangle(4)+ (smallTriangle(5)*Wc)/100;
        smallTriangle(5)= (smallTriangle(5)*Wa)/100;
        SPLIT(5)=0;
        %smallTriangle(5)=0;
 end
%  smallTriangle
%  ss=4
%  sum(smallTriangle)
 if SPLIT(7)==6 && SPLIT(8)~=7
      FI=asp(7)-180;
      [Wa,Wc]=VectorSplit90(FI);  
                
        smallTriangle(6)= smallTriangle(6)+(smallTriangle(7) *Wc)/100;
        smallTriangle(7)= (smallTriangle(7)*Wa)/100;
        SPLIT(7)=0;
        %smallTriangle(7)=0;
 end
% smallTriangle
% ss=5
% sum(smallTriangle)
 %%
 % for even cells direction anti-clockwise
 for i=2:2:6
 if SPLIT(i)==i-1 && SPLIT(i+1)~=i
   FI=((i-1)*45)-asp(i);
          [Wa,Wc]=VectorSplit45(FI); 
%             
                
        smallTriangle(i-1)= smallTriangle(i-1)+(smallTriangle(i) *Wa)/100;
        smallTriangle(i)= (smallTriangle(i)*Wc)/100;
        SPLIT(i)=0;
        %smallTriangle(i)=0;
% i
%  sum(smallTriangle)
 end   
 end
 
 if SPLIT(8)==7 && SPLIT(1)~=8
   FI=315-asp(8);
          [Wa,Wc]=VectorSplit45(FI); 
%             
                
        smallTriangle(7)= smallTriangle(7)+(smallTriangle(8) *Wa)/100;
        smallTriangle(8)= (smallTriangle(8)*Wc)/100;
        SPLIT(8)=0;
        %smallTriangle(i)=0;

 end   
 
%  smallTriangle
%  ss=6
%  sum(smallTriangle)
 %
 % for even cells clockwise direction
 
 if SPLIT(2)==3 && SPLIT(1)~=2
   
     FI=asp(2)-90;
      [Wa,Wc]=VectorSplit90(FI);  
%             A=FI*pi/180;
%         AA=(90-FI)*pi/180;
%         vector_a = (sin(A))/(sin(AA));
%         Wa=vector_a/(vector_a +1);
%         Wa=(fix(Wa*10000))/100;
%         Wc=100-Wa;
                
        smallTriangle(3)= smallTriangle(3)+(smallTriangle(2) *Wa)/100;
        smallTriangle(2)= (smallTriangle(2)*Wc)/100;
        SPLIT(2)=0;
        %smallTriangle(2)=0;
 end
% smallTriangle
% ss=7
%  sum(smallTriangle)
 
 if SPLIT(4)==5 && SPLIT(3)~=4
      FI=asp(4)-180;
      [Wa,Wc]=VectorSplit90(FI);  
                
        smallTriangle(5)= smallTriangle(5)+ (smallTriangle(4)*Wa)/100;
        smallTriangle(4)= (smallTriangle(4)*Wc)/100;
        SPLIT(4)=0;
        %smallTriangle(2)=0;
 end
%  smallTriangle
%  ss=8
%  sum(smallTriangle)
 
 if SPLIT(6)==7 && SPLIT(5)~=6
      FI=asp(6)-270;
      [Wa,Wc]=VectorSplit90(FI);  
                
        smallTriangle(7)= smallTriangle(7)+(smallTriangle(6) *Wa)/100;
        smallTriangle(6)= (smallTriangle(6)*Wc)/100;
        SPLIT(6)=0;
        %smallTriangle(6)=0;
 end
%  smallTriangle
%  ss=9
%  sum(smallTriangle)
 if SPLIT(8)==1 && SPLIT(7)~=8
      FI=asp(8);
      [Wa,Wc]=VectorSplit90(FI);  
                
        smallTriangle(1)= smallTriangle(1)+(smallTriangle(8) *Wa)/100;
        smallTriangle(8)= (smallTriangle(8)*Wc)/100;
        SPLIT(8)=0;
        %smallTriangle(8)=0;
 end
 
%  smallTriangle
%  ss=10
%  sum(smallTriangle)
SumSPLIT=sum(SPLIT);

 end %While sum(split)
 
 
%  if r==66 && c==293
%     P=989
%      asp
%      smallTriangle
%      Fidx
%      tarSec
%      %FPtemp
%      pause
%  end   

 
 
 
 
 
 %% Define cell sequanse and target sectors
 Cells=[1 2; 2 3; 3 4; 4 5; 5 6; 6 7; 7 8; 8 1];
%
tarSec=zeros(1,8);
for i=1:8
    if cellsmat(Cells(i,1))<cellsmat(10) && cellsmat(Cells(i,2))<cellsmat(10)
        tarSec(i)=20;
   elseif cellsmat(Cells(i,1))==cellsmat(10) && cellsmat(Cells(i,2))==cellsmat(10)
        tarSec(i)=20;
    
    elseif cellsmat(Cells(i,1))<=cellsmat(10) && cellsmat(Cells(i,2))>cellsmat(10)
        tarSec(i)=21;
    elseif cellsmat(Cells(i,1))<cellsmat(10) && cellsmat(Cells(i,2))>=cellsmat(10)
        tarSec(i)=21; 
        
    elseif cellsmat(Cells(i,1))>cellsmat(10) && cellsmat(Cells(i,2))<=cellsmat(10)
        tarSec(i)=22;
    elseif cellsmat(Cells(i,1))>=cellsmat(10) && cellsmat(Cells(i,2))<cellsmat(10)
        tarSec(i)=22;    
    
    end
end
 
%  
 
 %% Asign the triangle area to the cells
% 1. one cell is low then give the triangle area to that direction
FPtemp=zeros(8,1);  % empty matrix for temporary flow path

[Fidx]=smallTriangle>0; % mark index fo nonempty triangles

%To check if the flow is distributed to two higher cells
for i=1:8
    if Fidx(i)==1 % if nonempty triangle
 
        if (cellsmat(Cells(i,1))>cellsmat(10) && cellsmat(Cells(i,2))>cellsmat(10))
            % case of contradictive flow the flow traped between two high
            % cell due to contradictive
           contradictive_Tri=smallTriangle(i); %copy contradictive area
            Fidx(i)=0; %unmark contadictive index
            smallTriangle(i)=0; %zero contadictive triangle
            tempsum=sum(smallTriangle);
            for j=1:8
                if Fidx(j)==1 % if nonempty triangle
                    smallTriangle(j)=smallTriangle(j)+ ((contradictive_Tri* smallTriangle(j))/tempsum);
                end
                
            end
            
        end
    end
end

%%%%%%%%
%To check if the flow is distributed to a cell with same elevation as the
%center cell while there is a lower cell exist then the flow to the cell
%with the same elevation as center cell should be prevented and distribute
%the area to other cells
for i=1:8
    if Fidx(i)==1 % if nonempty triangle
        sumFidx=sum(Fidx);
        if  sumFidx>1 &&(cellsmat(Cells(i,1))>=cellsmat(10) && cellsmat(Cells(i,2))>=cellsmat(10))
            % case of contradictive flow the flow traped between two high
            % cell due to contradictive
           contradictive_Tri=smallTriangle(i); %copy contradictive area
            Fidx(i)=0; %unmark contadictive index
            smallTriangle(i)=0; %zero contadictive triangle
            tempsum=sum(smallTriangle);
            for j=1:8
                if Fidx(j)==1 % if nonempty triangle
                    smallTriangle(j)=smallTriangle(j)+ ((contradictive_Tri* smallTriangle(j))/tempsum);
                end
                
            end
            
        end
    end
end
%%%%%%%%


for i=1:8
    if Fidx(i)==1 % if nonempty triangle
        
        if tarSec(i)==21
           FPtemp (Cells(i,1))=FPtemp (Cells(i,1)) + (smallTriangle(i)*100); % if one is less
        end
        
        if tarSec(i)==22
           FPtemp (Cells(i,2))=FPtemp (Cells(i,2)) + (smallTriangle(i)*100); % if the other one is less 
        end
        
        
        % Split acording to the differnce in elevation 
        if tarSec(i)==20
 
            
%  if r==66 && c==293
%      p=1101
%     asp
%      smallTriangle
%      Fidx
%      tarSec
%      FPtemp
%      pause
%  end   
           
            
            
            
            
 %%          
%       a=(cellsmat(10)-cellsmat(Cells(i,1)));
%       b=(cellsmat(10)-cellsmat(Cells(i,2)));
 %% note disable this if you donot want to divid by square root 2
       %give the diagonal cells less flow by sqrt of 2 direction
%          if mod(i,2)==1 %odd number
%           b=b/sqrt(2); 
%         else
%             a=a/sqrt(2);
%          end
 %%
%         a1=(a/(a+b));
%         b1=b/(a+b);
%         FPtemp (Cells(i,1))=FPtemp (Cells(i,1)) + (a1* smallTriangle(i)*100);
%         FPtemp (Cells(i,2))=FPtemp (Cells(i,2)) + (b1* smallTriangle(i)*100);
        
 %% 
 
%         % split the flow according to vector split
%         %define the two cells that Fi is pointing to them
%         %FI is angle measured clockwise starting from the Y axis
%         
% FI2=-99;
% %odd forward
% if i==1 && asp(1)>=0 && asp(1) <90 % case 2 odd
% %Wa=100; Wc=0; %100 to cell 2
% FI2=asp(i);
% end
% if i==3 && asp(3)>=90 && asp(3) <180 %case 2 odd
% FI2=asp(i)-90;
% end
% if i==5 && asp(5)>=180 && asp(5) <270
% FI2=asp(i)-180;
% end
% if i==7 && asp(7) >=270 && asp(7) <360
% FI2=asp(i)-270;
% end
% 
% %even forward
% if i==2 && asp(2)>=90 && asp(2) <180 % case 6 even
% FI2=asp(i)-90;
%  end
% if i==4 && asp(4)>=180 && asp(4) <270 % case 5 even
%  FI2=asp(i)-180;
% end
% if i==6 && asp(6) >=270 && asp(6) <360
%  FI2=asp(i)-270;
% end
% if i==8 && asp(8) >=0 && asp(8) <90
% FI2=asp(i);
% end
% if FI2>=0 && FI2 <90
%         A=FI2*pi/180;
%         AA=(90-FI2)*pi/180;
%         vector_a = (sin(A))/(sin(AA));
%         Wa=vector_a/(vector_a +1);
%         Wa=0.5+Wa/2;
%         Wa=(fix(Wa*10000))/100;
%         Wc=100-Wa;
%  
% FPtemp(Cells(i,1))=FPtemp(Cells(i,1)) + (Wc * smallTriangle(i));
% FPtemp(Cells(i,2))=FPtemp(Cells(i,2)) + (Wa * smallTriangle(i));
% end
% 
% FI2=-99;
% %odd back
% if i==1 && asp(1)>=270 && asp(1) <360 % case 6 odd
% %Wa=0; Wc=100; %100 to cell 1
% FI2=asp(i)-270;
% end
% if i==3 && asp(3)>0 && asp(3) <90 % case 6 odd
% FI2=asp(i);
% end 
% if i==5 && asp(5) >=90 && asp(5) <180
% FI2=asp(i)-90;
%  end 
% if i==7 && asp(7)>=180 && asp(7) <270
% FI2=asp(i)-180;
% end
% if i==2 && asp(2)>=0 && asp(2) <90 % case 2 even
% %Wa=100; Wc=0;
% FI2=asp(i);
% end
% if i==4 && asp(4)>=90 && asp(4) <180 % case 3 even
% FI2=asp(i)-90;
%  end
% if i==6 && asp(6)>=180 && asp(6) <270
%  FI2=asp(i)-180;
%  end
% if i==8 && asp(8)>=270 && asp(8) <360
%  FI2=asp(i)-270;
% end
% if FI2>=0 && FI2 <90
%         A=FI2*pi/180;
%         AA=(90-FI2)*pi/180;
%         vector_a = (sin(A))/(sin(AA));
%         Wa=vector_a/(vector_a +1);
%         Wa=1-(Wa/2);
%         Wa=(fix(Wa*10000))/100;
%         Wc=100-Wa;
% FPtemp(Cells(i,1))=FPtemp(Cells(i,1)) + (Wa * smallTriangle(i));
% FPtemp(Cells(i,2))=FPtemp(Cells(i,2)) + (Wc * smallTriangle(i));     
% end     


%%
if mod(i,2)==1 %odd number
%odd back
if i==1 && asp(1)>=45 && asp(1) <=90 % case 2 odd
Wa=100; Wc=0; %100 to cell 2
end
if i==3 && asp(3)>=135 && asp(3) <=180 %case 2 odd
Wa=100; Wc=0;
end
if i==5 && asp(5)>=225 && asp(5) <=270
Wa=100; Wc=0;
end
if i==7 && asp(7) >=315 && asp(7) <=360
Wa=100; Wc=0;
 end

%odd foreard
if i==1 && asp(1)>=270 && asp(1) <=360 % case 6 odd
Wa=0; Wc=100; %100 to cell 1
end
if i==3 && asp(3)>=0 && asp(3) <=90 % case 6 odd
Wa=0; Wc=100;
end 
if i==5 && asp(5) >=90 && asp(5) <=180
Wa=0; Wc=100;
 end 
if i==7 && asp(7)>=180 && asp(7) <=270
Wa=0; Wc=100;
end

      FI2=(asp(i)-((i-1)*45));
      if FI2 >0 && FI2 <45
         
            A=FI2*pi/180;
        AA=(45-FI2)*pi/180;
        vector_a = (sin(A))/(sin(AA));
        Wa=vector_a/(vector_a +1);
        Wa=(fix(Wa*10000))/100;
        Wc=100-Wa;
      end
      
         FPtemp(Cells(i,1))=FPtemp(Cells(i,1)) + (Wc * smallTriangle(i));
        FPtemp(Cells(i,2))=FPtemp(Cells(i,2)) + (Wa * smallTriangle(i));
%        
%         i
%         asp(i)
%        FI2
%        A
%        AA
%        
%         Wcodd=Wc
%         Waodd=Wa
%          pause      
      
end
    
% if r==66 && c==293
%     P=1276
%     asp
%      smallTriangle
%      Fidx
%      tarSec
%      FPtemp
%      pause
%  end   


 if mod(i,2)==0 %even number
%even back
if i==2 && asp(2)>=0 && asp(2) <=45 % case 2 even
Wa=100; Wc=0;
end
if i==4 && asp(4)>=90 && asp(4) <=135 % case 3 even
 Wa=100; Wc=0;
 end
if i==6 && asp(6)>=180 && asp(6) <=225
 Wa=100; Wc=0;
 end
if i==8 && asp(8)>=270 && asp(8) <=315
 Wa=100; Wc=0;
end


%even forward
if i==2 && asp(2)>=90 && asp(2) <=180 % case 6 even
Wa=0; Wc=100;
 end
if i==4 && asp(4)>=180 && asp(4) <=270 % case 5 even
 Wa=0; Wc=100;
end
if i==6 && asp(6) >=270 && asp(6) <=360
 Wa=0; Wc=100;
end

if i==8 && asp(8) >=0 && asp(8) <=90
Wa=0; Wc=100;
end

 if r==66 && c==293

 end
FI2=((i*45)- asp(i));
  if FI2 >0 && FI2 <45        
          A=FI2*pi/180;
        AA=(45-FI2)*pi/180;
        vector_a = (sin(A))/(sin(AA));
        Wa=vector_a/(vector_a +1);
        Wa=(fix(Wa*10000))/100;
        Wc=100-Wa;
  end      
        
        FPtemp(Cells(i,1))=FPtemp(Cells(i,1)) + (Wa * smallTriangle(i));
        FPtemp(Cells(i,2))=FPtemp(Cells(i,2)) + (Wc * smallTriangle(i));
 
    
%         if r==66 && c==293
%     P=1327
%    i
%         asp(i)
%        FI2
%        A
%        AA
%         Wceven=Wc
%         Waeven=Wa
%         pause
%     
%     
%     asp
%      smallTriangle
%      Fidx
%      tarSec
%      FPtemp
%      pause
%        end  
        
        
        
 end
% if r==66 && c==293
%     P=1327
%     asp
%      smallTriangle
%      Fidx
%      tarSec
%      FPtemp
%      pause
%  end   






%         if mod(i,2)==1 %odd number
%             FI2=(asp(i)-((i-1)*45));
%             
%             A=FI2*pi/180;
%         AA=(45-FI2)*pi/180;
%         vector_a = (sin(A))/(sin(AA));
%         Wa=vector_a/(vector_a +1);
%         Wa=(fix(Wa*10000))/100;
%         Wc=100-Wa;
%         
% %         FPtemp(Cells(i,1))=Wc * smallTriangle(i);
% %         FPtemp(Cells(i,2))=Wa * smallTriangle(i);
%         FPtemp(Cells(i,1))=FPtemp(Cells(i,1)) + (Wc * smallTriangle(i));
%         FPtemp(Cells(i,2))=FPtemp(Cells(i,2)) + (Wa * smallTriangle(i));
%        
%         i
%         asp(i)
%        FI2
%        A
%        AA
%        
%         Wcodd=Wc
%         Waodd=Wa
%          pause 
%         end
%         
%         if mod(i,2)==0
%           FI2=((i*45)- asp(i));
%           
%           A=FI2*pi/180;
%         AA=(45-FI2)*pi/180;
%         vector_a = (sin(A))/(sin(AA));
%         Wa=vector_a/(vector_a +1);
%         Wa=(fix(Wa*10000))/100;
%         Wc=100-Wa;
%         
%         
%         FPtemp(Cells(i,1))=FPtemp(Cells(i,1)) + (Wa * smallTriangle(i));
%         FPtemp(Cells(i,2))=FPtemp(Cells(i,2)) + (Wc * smallTriangle(i));
%         asp(i)
%        FI2
%        A
%        AA
%         Wceven=Wc
%         Waeven=Wa
%         pause
%         end
        
        end
                        
     end
                     
end

if cm==1
FPtemp=zeros(8,1);
FPtemp(OneCellIndex) = 100;
end

% if r==66 && c==293
%     P=1393
%     asp
%      smallTriangle
%      Fidx
%      tarSec
%      FPtemp
%      pause
%  end   
   
    FP1(r,c)=FPtemp(1);
    FP2(r,c)=FPtemp(2);
    FP3(r,c)=FPtemp(3);
    FP4(r,c)=FPtemp(4);
    FP5(r,c)=FPtemp(5);
    FP6(r,c)=FPtemp(6);
    FP7(r,c)=FPtemp(7);
    FP8(r,c)=FPtemp(8);
    %check the check matrix with 1 instead of zero to mark the done cell    
    Check_Matrix(r,c)=1;
    Remain_cells=Remain_cells-1
  
     end % end if classfication
   
  if Classification_matrix(r,c)==2  % Sink
    %give zero to all directions if the class is sink
    FP1(r,c)=0;
    FP2(r,c)=0;
    FP3(r,c)=0;
    FP4(r,c)=0;
    FP5(r,c)=0;
    FP6(r,c)=0;
    FP7(r,c)=0;
    FP8(r,c)=0;
    %    
    Check_Matrix(r,c)=1;
     Remain_cells=Remain_cells-1  
   end %end of sink  
    
end %end for cols
end %end for rows
%

%
%% split the flow according to vector split
%define the two cells that Fi is pointing to them
        %FI is angle measured clockwise starting from the Y axis 
        %calculate to which sector this FI belong to
       
