%Matlab function to calculate the slope using trend surface (the form of
%the surface)
%SlopeAspect.m 
%Abdulghani 20011-10-21

function [Slope,Aspect]=SlopeAspect(ZT,cellsize,nrows,ncols)

% To test slope calculation using trend surface _form-Based
%Create coordinate matrix same for all cells/resolutions. the coordinate
%for the nine cells as cell1 [1 x y x2 y2 x.y] x2 and y2 mean x square
 %                     cell2 [1 x y x2 y2 x.y]
 %                     cell .................
 %                     cell9 [1 x y x2 y2 x.y]
 
 
   B=[1 -1  1  1  1 -1;
      1  0  1  0  1  0;
      1  1  1  1  1  1;
      1  1  0  1  0  0;
      1  1 -1  1  1 -1;
      1  0 -1  0  1  0;
      1 -1 -1  1  1  1;
      1 -1  0  1  0  0;
      1  0  0  0  0  0];
  B=B*cellsize;
  %coordinates and numbering a 3*3 window cells
    %-------------------------
    %| 1  |   2   |   3   |
    %| 8  |   10  |   4   |
    %| 7  |   6   |   5   |
    %-------------------------
    

Slope=zeros(nrows,ncols);     %create empty matrix with zeros
Aspect=zeros(nrows,ncols);     %create empty matrix with zeros
Slope(Slope == 0)=NaN;          %fill the matrix with NaNs
Aspect(Aspect == 0)=NaN;
for r=2:nrows-1                  %Go through all cells except the borders
for c=2:ncols-1
    
    %create a window of 9 cells around the current cell
    Cell1=ZT(r-1,c-1); Cell2=ZT(r-1,c); Cell3=ZT(r-1,c+1);Cell4=ZT(r,c+1);Cell5=ZT(r+1,c+1);
    Cell6=ZT(r+1,c); Cell7=ZT(r+1,c-1); Cell8=ZT(r,c-1); Cell10=ZT(r,c);
    
    %Putting all 9 cells in a vector matrix including the centre cell
    Cellsmat=[Cell1; Cell2; Cell3; Cell4; Cell5; Cell6; Cell7; Cell8; Cell10];
    %if all cells are positive (have values)
    if Cell1>0 && Cell2>0 && Cell3>0 && Cell4>0 && Cell5>0 && Cell6>0 && Cell7>0 && Cell8>0 && Cell10>0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% the cell size is taken in to account up when calculating B
%use least square method to solve the over determined  system and
    %calculate the 5 constants 
    u=B\Cellsmat; %solve the equation u=B.I. eqn 4 in the paper   
    %Slope calculation
    Slope(r,c) = (atan(((u(2)^2)+((u(3)^2)))^0.5))*180/pi;%Slope Trend surface 
    
% Aspect calculation
%calculate the main drainage direction (FI) using eqn 6 in the paper
    if u(3) > 0 %a2
        FI=(atan(u(2)/u(3))*180/pi) + 180;
    elseif u(3) < 0 && u(2) <= 0 
        FI=(atan(u(2)/u(3))*180/pi);
    elseif u(3) < 0 && u(2) > 0 
        FI=(atan(u(2)/u(3))*180/pi) + 360;
    elseif u(3) == 0 && u(2) > 0 
        FI=270;
    elseif u(3) == 0 && u(2) < 0 
        FI=90;
    end
    
    Aspect(r,c)=FI;
    %
    end
    if Slope(r,c)==0
        Slope(r,c)= 0.01;
    end
end
end
% end of function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%