% Function to calculate the depth of thawing (Active Layer thickness ALT)
%Abdulghani Hasan,  Feb. 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%                    %%                 %%         %%%%%%%%%%%%%%%%%%%%%%
%                  %% %%                %%         %%%%%%%%%%%%%%%%%%%%%% 
%                 %%   %%               %%                  %%
%                %%     %%              %%                  %%
%               %%       %%             %%                  %%
%              %%         %%            %%                  %%
%             %%           %%           %%                  %%
%            %%%%%%%%%%%%%%%%%          %%                  %%
%           %%               %%         %%                  %%
%          %%                 %%        %%                  %%
%         %%                   %%       %%                  %%
%        %%                     %%      %%                  %%
%       %%                       %%     %%%%%%%%%%%%%%%%%%  %%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [TWI,ALT]=TWIALT(Classification_matrix,DA,nrows,ncols,cellsize,Slope,Tds)

ALT=zeros(nrows,ncols);
ALT(isnan(Classification_matrix))= NaN;
TWI=zeros(nrows,ncols);
TWI(isnan(Classification_matrix))= NaN;
P = 31536000; % 31536000 the period of temperature oscillations (1 year)(s)
Aa = 45; % 13 c is the annual amplitude of the air temperature (mean MaxT - mean MinT)/2
%%%%%%%%%%%
Ta =0; %-1.0 c is mean annual air temperature
%%%%%%%%%%
%Ta = -Ta;
Zsn = 0.5; % 0.5 m thickness of snow cover
Ksn = 0.000000507; % 0.000000507 thermal diffusivity of snow
TAO1 = (178/365)*P;  % 21536000 s the duration of cold period
TAO2 = (187/365)*P; % 10000000 s the duration of warm period
Zveg = 0.06; % 0.1 m is the thickness of vegetation cover (m)
Ktveg = 0.00000003; %Represent the thermal diffusivity of vegetation
                    %in the thawed state (m2/s).
Kfveg = 0.000000075; %Represent the thermal diffusivity of vegetation
               %in the frozen state (m2/s).
DeltaAsn =0.5; % 0.2 m change in the amplitude of snow cover depth
C = 50000; % (30000) is the volumetric heat capacity of soil in the thawed state (J/m3 C)
%Psn = 10000000; %????? 100000 is the period of the temperature wave, adjusted for snow melt (s)
Qph = 4000000; %???? 1000000 is the volumetric latent heat of phase change (J/m3).
DeltaTsn = Aa*(1-(exp(-Zsn*(pi/(P*Ksn))^.5)));
Tveg = Ta + DeltaTsn;
Aveg = Aa - DeltaAsn;
% differences in average temperature at the surface of the plant cover and
%under it during the cold periods, (C).
DeltaA1 = (Aveg - Tveg)* (1-exp(-Zveg*(pi/(Kfveg*2*TAO1))^.5));

% differences in average temperature at the surface of the plant cover and
%under it during the warm periods, (C).
DeltaA2 = (Aveg + Tveg)* (1-exp(-Zveg*(pi/(Ktveg*2*TAO2))^.5));

%Delat calculations
DeltaAveg = ((DeltaA1 * TAO1 + DeltaA2 * TAO2)/P);

DeltaTveg = ((DeltaA1 * TAO1 - DeltaA2 * TAO2)/P)* pi/2;

Ts = Ta + DeltaTsn + DeltaTveg %Annual temperature at soil surface (C)

As = Aa + DeltaAsn - DeltaAveg  %Annual amplitude of soil surface temp.(C)
pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
for r=2:nrows-1
for c=2:ncols-1
if Classification_matrix(r,c)>0
 if Slope(r,c)<0.001 %if the slope is too small this will result flat
    Slope(r,c)=0.001;
end   
TWI(r,c)= log((DA(r,c)*cellsize^2)/(tan(Slope(r,c)*pi/180))); %using slope here could be change by maximum slope to the cell in flow direction
if TWI(r,c)<0 %if the slope is heigh and the DA is small this will result dry soil(TWI=0)
    TWI(r,c)=0;
end
TWIm=(TWI(r,c)/100);%/30 ??????????????

LAMDAt = 0.34 + TWIm; %*.335(1.0 - 0.2); % = 0.85 is the thermal conductivity of soil in the thawed state ((W/m C),

LAMDAf = 0.97 + TWIm ;%*.87(1.45 - 0.35);% 2.65 represents the thermal conductivity of frozen

%LAMDAt = 0.13 + TWIm*(1.0 - 0.2); % = 0.85 is the thermal conductivity of soil in the thawed state ((W/m C),

%LAMDAf = 0.35 + TWIm*(1.45 - 0.35);% 2.65 represents the thermal conductivity of frozen soil (W/mC).

% LAMDAt if numerater > 0 and equal LAMDAf if numerator < o

%%The mean annual temperature at the depth of seasonal thaw can be calculated using
%the semi-empirical equation (Garagulia, 1985, p.49):

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%use this if you have distributed Temp matrix Tds
% Tzb11 = 0.5*(Tds(r,c))*(LAMDAf+LAMDAt);
% Tzb12 = As*(LAMDAf-LAMDAt)/pi;
% Tzb21 = ((Tds(r,c))/As)*asin((Tds(r,c))/As);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
%use this if temp is not distributed
Tzb11 = 0.5*(Ts)*(LAMDAf+LAMDAt);
Tzb12 = As*(LAMDAf-LAMDAt)/pi;
Tzb21 = ((Ts)/As)*asin((Ts)/As);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Tzb22 = (1-(pi^2)/As^2)^0.5;
Tzb2 = Tzb21 + Tzb22;
Tzb = Tzb11 + Tzb12 * Tzb2;

if Tzb >= 0 % calculate LAMDAstar depending on the upper portion numerator
    
LAMDA = LAMDAt;
end
if Tzb < 0
    
LAMDA = LAMDAf;
end


Tz =Tzb/LAMDA; %The mean annual temperature at the depth of seasonal thaw

ALFA = 2*C*As/Qph;

BETA = 2*C*Tz/Qph;

GAMA = ALFA + BETA;

SEGMA = log((ALFA+1)/(BETA+1));

B=SEGMA+(SEGMA/(2*GAMA))-(GAMA/(2*SEGMA))-(GAMA/2);

D= SEGMA+(SEGMA*GAMA)+(GAMA-SEGMA^2)-(SEGMA^2/GAMA);

Zstar =B + (B^2 + D)^0.5;

Za = Zstar * ((LAMDA * P )/(pi* C))^0.5; %thawing depth
ALT(r,c)=Za;

if r==643 && c==188
    c
    Tzb
    LAMDAt
    LAMDAf
    LAMDA
    Tz
    ALFA
    BETA
    GAMA
    SEGMA
    B
    D
    Zstar
    Za
    
    pause
end
if r==643 && c==187
    c
    Tzb
    LAMDAt
    LAMDAf
    LAMDA
    Tz
    ALFA
    BETA
    GAMA
    SEGMA
    B
    D
    Zstar
    Za
    pause
end







end
end
end